# """Unit tests for __version__.py."""

# import CAF


# def test_package_version():
#     """Ensure the package version is defined and not set to the initial
#     placeholder."""
#     assert hasattr(CAF, "__version__")
#     assert CAF.__version__ != "0.0.0"
