#!/usr/bin/env python
##############################################################################
#
# (c) 2025 Sangjoon Lee.
# All rights reserved.
#
# File coded by: Sangjoon Lee, Emil Jaffal, Danila Shiryaev, Nikhil Kumar Barua
#
# See GitHub contributions for a more detailed list of contributors.
# https://github.com/bobleesj/composition-analyzer-featurizer/graphs/contributors
#
# See LICENSE.rst for license information.
#
##############################################################################
"""Python package for generating compositional features for binary, ternary,
and quaternary formulas with Oliynyk elemental properties."""

# package version
# from CAF.version import __version__

# silence the pyflakes syntax checker
# assert __version__ or True

# End of file
